Type: Package
Package: later2
Title: Utilities for Scheduling Functions to Execute Later with Event
        Loops
Version: 0.1
Authors@R: c(
    person("Winston", "Chang", , "winston@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-1576-2126")),
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("Charlie", "Gao", , "charlie.gao@posit.co", role = "aut",
           comment = c(ORCID = "0000-0002-0750-061X")),
    person("Posit Software, PBC", role = c("aut", "cph"),
           comment = c(ROR = "03wc8by49")),
    person("Marcus", "Geelnard", role = c("ctb", "cph"),
           comment = "TinyCThread library, https://tinycthread.github.io/"),
    person("Evan", "Nemerson", role = c("ctb", "cph"),
           comment = "TinyCThread library, https://tinycthread.github.io/"),
    person(
        given = "Mauricio", family = "Vargas Sepulveda", role = c("aut", "cre"),
        email = "m.vargas.sepulveda@gmail.com",
        comment = c(ORCID = "0000-0003-1017-7574"))
  )
Description: Executes arbitrary R or C functions some time after the current
    time, after the R execution stack has emptied. The functions are scheduled
    in an event loop. This is a derived work from the 'later' package aiming
    to reduce the number of dependencies.
License: Apache License 2.0
URL: https://github.com/pachadotdev/later2
BugReports: https://github.com/pachadotdev/later2/issues
Depends: R (>= 3.5)
Suggests: tinytest, parallel, litedown
LinkingTo: cpp4r
Encoding: UTF-8
VignetteBuilder: litedown
NeedsCompilation: yes
Packaged: 2026-08-23 18:00:28 UTC; pacha
Author: Winston Chang [aut] (ORCID: <https://orcid.org/0000-0002-1576-2126>),
  Joe Cheng [aut],
  Charlie Gao [aut] (ORCID: <https://orcid.org/0000-0002-0750-061X>),
  Posit Software, PBC [aut, cph] (ROR: <https://ror.org/03wc8by49>),
  Marcus Geelnard [ctb, cph] (TinyCThread library,
    https://tinycthread.github.io/),
  Evan Nemerson [ctb, cph] (TinyCThread library,
    https://tinycthread.github.io/),
  Mauricio Vargas Sepulveda [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1017-7574>)
Maintainer: Mauricio Vargas Sepulveda <m.vargas.sepulveda@gmail.com>
Repository: CRAN
Date/Publication: 2026-09-03 12:50:02 UTC
