# Generated by tinyroxygen: do not edit by hand
useDynLib(later2, .registration=TRUE)
S3method(format, event_loop)
S3method(format, promise)
S3method(print, event_loop)
S3method(print, promise)
export(catch)
export(create_loop)
export(current_loop)
export(destroy_loop)
export(exists_loop)
export(finally)
export(global_loop)
export(is.promise)
export(later)
export(later_fd)
export(loop_empty)
export(next_op_secs)
export(promise)
export(run_now)
export(then)
export(with_loop)
export(with_temp_loop)
