# Generated by cpp4r: do not edit by hand

testCallbackOrdering <- function() {
	invisible(.Call(`_later2_testCallbackOrdering`))
}

log_level <- function(level) {
	.Call(`_later2_log_level`, level)
}

using_ubsan <- function() {
	.Call(`_later2_using_ubsan`)
}

execLater_fd <- function(callback, readfds, writefds, exceptfds, timeoutSecs, loop_id) {
	.Call(`_later2_execLater_fd`, callback, readfds, writefds, exceptfds, timeoutSecs, loop_id)
}

fd_cancel <- function(xptr) {
	.Call(`_later2_fd_cancel`, xptr)
}

setCurrentRegistryId <- function(id) {
	invisible(.Call(`_later2_setCurrentRegistryId`, id))
}

getCurrentRegistryId <- function() {
	.Call(`_later2_getCurrentRegistryId`)
}

deleteCallbackRegistry <- function(loop_id) {
	.Call(`_later2_deleteCallbackRegistry`, loop_id)
}

notifyRRefDeleted <- function(loop_id) {
	.Call(`_later2_notifyRRefDeleted`, loop_id)
}

createCallbackRegistry <- function(id, parent_id) {
	invisible(.Call(`_later2_createCallbackRegistry`, id, parent_id))
}

existsCallbackRegistry <- function(id) {
	.Call(`_later2_existsCallbackRegistry`, id)
}

list_queue_ <- function(id) {
	.Call(`_later2_list_queue_`, id)
}

execCallbacks <- function(timeoutSecs, runAll, loop_id) {
	.Call(`_later2_execCallbacks`, timeoutSecs, runAll, loop_id)
}

idle <- function(loop_id) {
	.Call(`_later2_idle`, loop_id)
}

ensureInitialized <- function() {
	invisible(.Call(`_later2_ensureInitialized`))
}

execLater <- function(callback, delaySecs, loop_id) {
	.Call(`_later2_execLater`, callback, delaySecs, loop_id)
}

cancel <- function(callback_id_s, loop_id) {
	.Call(`_later2_cancel`, callback_id_s, loop_id)
}

nextOpSecs <- function(loop_id) {
	.Call(`_later2_nextOpSecs`, loop_id)
}

new_weakref <- function(x) {
	.Call(`_later2_new_weakref`, x)
}

wref_key <- function(x) {
	.Call(`_later2_wref_key`, x)
}
